// Copyright 2000-2005 the Contributors, as shown in the revision logs.
// Licensed under the Apache Public Source License 2.0 ("the License").
// You may not use this file except in compliance with the License.

package org.ibex.jinetd;
import org.ibex.util.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.lang.reflect.*;

public class Host extends TreeClassLoader {

    public final String hostname;
    public final String path;

    private static final Hashtable hosts = new Hashtable();
    private static boolean initted = false;

    public static Host getHost(String hostname) { return (Host)hosts.get(hostname); }
    public static Enumeration enumerateHosts() { return hosts.elements(); }

    public Host(String path, String hostname, ClassLoader pcl) {
        super(new File(path + File.separatorChar + "WEB-INF"), pcl);
        this.hostname = hostname;
        this.path = path;
    }

    public static void init() {
        if (initted) return;
        initted = true;
        init(org.ibex.jinetd.Main.ROOT + "/host", "", 0, Main.getRootClassLoader());
    }
    public static void init(String path, String host, int depth, ClassLoader pcl) {
	if (!new File(path).exists() || !new File(path).isDirectory()) return;
        try {
	    Log.warn(Host.class, "resolving " + host);
            InetAddress ia = InetAddress.getByName(host);
            String pad = "";
            while(pad.length() + host.length() < 30) pad += " ";
            if (ia.equals(InetAddress.getLocalHost())) {
                Log.info(Main.class, pad + host + " => " + path);
                hosts.put(host, pcl = new Host(path, host, pcl));
            }
        } catch (UnknownHostException e) {
            if (depth >= 3) return;
        }
        String[] subdirs = new File(path).list();
        if (subdirs != null)
            for(int i=0; i<subdirs.length; i++)
                init(path + File.separatorChar + subdirs[i], host.equals("") ? subdirs[i] : subdirs[i]+"."+host, depth+1, pcl);
    }

}
