// Copyright 2000-2005 the Contributors, as shown in the revision logs.
// Licensed under the Apache Public Source License 2.0 ("the License").
// You may not use this file except in compliance with the License.

package org.ibex.jinetd;
import org.ibex.util.*;
import org.ibex.io.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.util.zip.*;

// Feature: port-level redirects
public class Port extends TreeClassLoader {

    final ThreadPool tp;
    final InetAddress bindTo;
    final int port;

    private static Hash cache = new Hash();
    public static Port newPort(String path, InetAddress bindTo, int port, ThreadPool tp) throws IOException {
        String canonical = new File(path).getCanonicalPath();
        Port p = (Port)cache.get(canonical);
        if (p == null) cache.put(canonical, p = new Port(path, bindTo, port, tp));
        else Log.warn(Port.class, "   sharing " + bindTo+":"+port+" -> "+ (p.bindTo+":"+p.port));
        p.spawn(bindTo, port);
        return p;
    }

    void spawn(InetAddress bindTo, int port) { new PortThread(bindTo, port).start(); }
    private static final ClassLoader parentLoader = Main.getRootClassLoader();
    private Port(String path, InetAddress bindTo, int port, ThreadPool tp) { 
        super(new File(path), parentLoader);
        this.bindTo = bindTo; this.port = port; this.tp = tp; }

    void dispatch(final Connection conn) throws Exception {
        tp.appendTask(new Runnable() { public void run() {
            String local = conn.getLocalAddress() + ":" + conn.getLocalPort();
            String remote = conn.getRemoteHostname() + ":" + conn.getRemotePort();
            try {
                // FIXME FIXME
                Class c = loadClass("org.ibex.mail.Main");
                if (c == null) throw new RuntimeException("couldn't find listener");
                Log.info("["+local+"]", "connection from " + remote + " => " + c.getName() + ":"+conn.getLocalPort());
                Log.clearnotes();
                Thread.currentThread().setContextClassLoader(Port.this);
                Listener l = (Listener)c.newInstance();
                l.accept(conn);
            } catch (org.ibex.io.Stream.EOF eof) {
                Log.warn(this, "end of stream reached handling connection from " +
                         conn.getRemoteHostname() + ":" + conn.getRemotePort());
            } catch (Exception e) {
                Log.error(this, e);
                conn.close();
            } finally {
                conn.close();
            }
        } });
    }

    private class PortThread extends Thread {
        InetAddress bindTo;
        int port;
        public PortThread(InetAddress bindTo, int port) { this.bindTo = bindTo; this.port = port; }
        public void run() {
            try {
                Log.warn(this, "Now listening on address " + (bindTo==null?"all interfaces":bindTo.toString()) + ", port " + port);
                ServerSocket ss = bindTo == null ? new ServerSocket(port) : new ServerSocket(port, 0, bindTo);
                for(Socket s = ss.accept(); ; s = ss.accept()) try {
                    dispatch(new Connection(s, Main.defaultDomain));
                } catch (Exception e) { Log.warn(Port.class, e); }
            } catch (Exception e) { Log.error(Port.class, e);
            } catch (Throwable t) {
                Log.error(this, "serious error, aborting VM");
                Log.error(this, t);
                Main.reboot();
            }
        }
    }
}
